@@ -54,7 +54,7 @@ def login(request, template_name="lfs/checkout/login.html"):
     if request.user.is_authenticated():
         return HttpResponseRedirect(reverse("lfs_checkout"))
 
-    shop = lfs.core.utils.get_default_shop()
+    shop = lfs.core.utils.get_default_shop(request)
 
     # If only anonymous checkout allowed we don't want to show this view at all.
     if shop.checkout_type == CHECKOUT_TYPE_ANON:
@@ -67,7 +67,6 @@ def login(request, template_name="lfs/checkout/login.html"):
 
     if request.POST.get("action") == "login":
         login_form = AuthenticationForm(data=request.POST)
-        login_form.fields["username"].label = _(u"E-Mail")
         if login_form.is_valid():
             from django.contrib.auth import login
             login(request, login_form.get_user())
@@ -108,7 +107,7 @@ def login(request, template_name="lfs/checkout/login.html"):
 def checkout_dispatcher(request):
     """Dispatcher to display the correct checkout form
     """
-    shop = lfs.core.utils.get_default_shop()
+    shop = lfs.core.utils.get_default_shop(request)
     cart = cart_utils.get_cart(request)
 
     if cart is None or not cart.get_items():
@@ -144,7 +143,7 @@ def cart_inline(request, template_name="lfs/checkout/checkout_cart_inline.html")
 
     discounts = lfs.discounts.utils.get_valid_discounts(request)
     for discount in discounts:
-        cart_price = cart_price - discount["price"]
+        cart_price = cart_price - discount["price_gross"]
         cart_tax = cart_tax - discount["tax"]
 
     # Voucher
@@ -158,12 +157,12 @@ def cart_inline(request, template_name="lfs/checkout/checkout_cart_inline.html")
         voucher_message = MESSAGES[6]
     else:
         lfs.voucher.utils.set_current_voucher_number(request, voucher_number)
-        is_voucher_effective, voucher_message = voucher.is_effective(request, cart)
+        is_voucher_effective, voucher_message = voucher.is_effective(cart)
         if is_voucher_effective:
             display_voucher = True
-            voucher_value = voucher.get_price_gross(request, cart)
+            voucher_value = voucher.get_price_gross(cart)
             cart_price = cart_price - voucher_value
-            voucher_tax = voucher.get_tax(request, cart)
+            voucher_tax = voucher.get_tax(cart)
         else:
             display_voucher = False
             voucher_value = 0
@@ -171,9 +170,12 @@ def cart_inline(request, template_name="lfs/checkout/checkout_cart_inline.html")
 
     cart_items = []
     for cart_item in cart.get_items():
+        product = cart_item.product
+        quantity = product.get_clean_quantity(cart_item.amount)
         cart_items.append({
             "obj": cart_item,
-            "product": cart_item.product,
+            "quantity" : quantity,
+            "product": product,
             "product_price_net": cart_item.get_price_net(request),
             "product_price_gross": cart_item.get_price_gross(request),
             "product_tax": cart_item.get_tax(request),
@@ -203,7 +205,7 @@ def one_page_checkout(request, checkout_form=OnePageCheckoutForm,
     """
     # If the user is not authenticated and the if only authenticate checkout
     # allowed we rediret to authentication page.
-    shop = lfs.core.utils.get_default_shop()
+    shop = lfs.core.utils.get_default_shop(request)
     if request.user.is_anonymous() and \
        shop.checkout_type == CHECKOUT_TYPE_AUTH:
         return HttpResponseRedirect(reverse("lfs_checkout_login"))
@@ -211,22 +213,37 @@ def one_page_checkout(request, checkout_form=OnePageCheckoutForm,
     customer = customer_utils.get_or_create_customer(request)
     if request.method == "POST":
         form = checkout_form(request.POST)
-
         toc = True
 
         if shop.confirm_toc:
             if "confirm_toc" not in request.POST:
                 toc = False
-                if form._errors is None:
-                    form._errors = {}
-                form._errors["confirm_toc"] = _(u"Please confirm our terms and conditions")
+                if form.errors is None:
+                    form.errors = {}
+                form.errors["confirm_toc"] = _(u"Please confirm our terms and conditions")
 
-        if toc and form.is_valid():
+        # Validate invoice address
+        prefix="invoice"
+        country_iso = request.POST.get(prefix + "-country", shop.default_country.code)
+        form_class = form_factory(country_iso)
+        invoice_form = form_class(request.POST, prefix=prefix)
+
+        # Validate shipping address
+        valid_shipping_form = True
+        if not request.POST.get("no_shipping"):
+            prefix="shipping"
+            country_iso = request.POST.get(prefix + "-country", shop.default_country.code)
+            form_class = form_factory(country_iso)
+            shipping_form = form_class(request.POST, prefix=prefix)
+            valid_shipping_form = shipping_form.is_valid()
+
+        if form.is_valid() and invoice_form.is_valid() and valid_shipping_form and toc:
             # save invoice details
             customer.selected_invoice_address.firstname = request.POST.get("invoice_firstname")
             customer.selected_invoice_address.lastname = request.POST.get("invoice_lastname")
             customer.selected_invoice_address.phone = request.POST.get("invoice_phone")
             customer.selected_invoice_address.email = request.POST.get("invoice_email")
+            customer.selected_invoice_address.company_name = request.POST.get("invoice_company_name")
 
             # Create or update invoice address
             valid_invoice_address = save_address(request, customer, INVOICE_PREFIX)
@@ -242,6 +259,7 @@ def one_page_checkout(request, checkout_form=OnePageCheckoutForm,
                     customer.selected_shipping_address.lastname = request.POST.get("shipping_lastname")
                     customer.selected_shipping_address.phone = request.POST.get("shipping_phone")
                     customer.selected_shipping_address.email = request.POST.get("shipping_email")
+                    customer.selected_shipping_address.company_name = request.POST.get("shipping_company_name")
 
                     valid_shipping_address = save_address(request, customer, SHIPPING_PREFIX)
 
@@ -267,22 +285,18 @@ def one_page_checkout(request, checkout_form=OnePageCheckoutForm,
                     # process the payment method ...
                     result = lfs.payment.utils.process_payment(request)
 
-                    next_url = None
-                    if result["accepted"] == True:
-                        return HttpResponseRedirect(
-                            result.get("next-url", reverse("lfs_thank_you")))
+                    if result["accepted"]:
+                        return HttpResponseRedirect(result.get("next_url", reverse("lfs_thank_you")))
                     else:
                         if "message" in result:
-                            form._errors[result.get("message-position")] = result.get("message")
+                            form._errors[result.get("message_location")] = result.get("message")
         else:  # form is not valid
             # save invoice details
             customer.selected_invoice_address.firstname = request.POST.get("invoice_firstname")
             customer.selected_invoice_address.lastname = request.POST.get("invoice_lastname")
             customer.selected_invoice_address.phone = request.POST.get("invoice_phone")
             customer.selected_invoice_address.email = request.POST.get("invoice_email")
-
-            # Create or update invoice address
-            save_address(request, customer, INVOICE_PREFIX)
+            customer.selected_invoice_address.company_name = request.POST.get("invoice_company_name")
 
             # If the shipping address differs from invoice firstname we create
             # or update the shipping address.
@@ -292,10 +306,9 @@ def one_page_checkout(request, checkout_form=OnePageCheckoutForm,
                 customer.selected_shipping_address.lastname = request.POST.get("shipping_lastname")
                 customer.selected_shipping_address.phone = request.POST.get("shipping_phone")
                 customer.selected_shipping_address.email = request.POST.get("shipping_email")
+                customer.selected_shipping_address.company_name = request.POST.get("shipping_company_name")
                 customer.save()
 
-                save_address(request, customer, SHIPPING_PREFIX)
-
             # Payment method
             customer.selected_payment_method_id = request.POST.get("payment_method")
 
@@ -324,6 +337,7 @@ def one_page_checkout(request, checkout_form=OnePageCheckoutForm,
             "invoice_phone": invoice_address.phone,
             "invoice_email": invoice_address.email,
             "invoice_country": invoice_address.country,
+            "invoice_company_name": invoice_address.company_name,
         })
         shipping_address = customer.selected_shipping_address
         initial.update({
@@ -331,7 +345,8 @@ def one_page_checkout(request, checkout_form=OnePageCheckoutForm,
             "shipping_lastname": shipping_address.lastname,
             "shipping_phone": shipping_address.phone,
             "shipping_email": shipping_address.email,
-            "no_shipping": False,
+            "shipping_company_name": shipping_address.company_name,
+            "no_shipping": True,
         })
         form = checkout_form(initial=initial)
     cart = cart_utils.get_cart(request)
@@ -348,8 +363,8 @@ def one_page_checkout(request, checkout_form=OnePageCheckoutForm,
     valid_payment_methods = lfs.payment.utils.get_valid_payment_methods(request)
     valid_payment_method_ids = [m.id for m in valid_payment_methods]
 
-    display_bank_account = any([pm.type == lfs.payment.settings.PM_BANK for pm in valid_payment_methods])
-    display_credit_card = any([pm.type == lfs.payment.settings.PM_CREDIT_CARD for pm in valid_payment_methods])
+    display_bank_account = DIRECT_DEBIT in valid_payment_method_ids
+    display_credit_card = CREDIT_CARD in valid_payment_method_ids
 
     response = render_to_response(template_name, RequestContext(request, {
         "form": form,
@@ -362,6 +377,7 @@ def one_page_checkout(request, checkout_form=OnePageCheckoutForm,
         "display_bank_account": display_bank_account,
         "display_credit_card": display_credit_card,
         "voucher_number": lfs.voucher.utils.get_current_voucher_number(request),
+        "settings": settings,
     }))
 
     return response
@@ -379,8 +395,10 @@ def thank_you(request, template_name="lfs/checkout/thank_you_page.html"):
     """Displays a thank you page ot the customer
     """
     order = request.session.get("order")
+    pay_link = order.get_pay_link(request) if order else None
     return render_to_response(template_name, RequestContext(request, {
         "order": order,
+        "pay_link": pay_link,
     }))
 
 
