@@ -1,3 +1,4 @@
+# django imports
 from django import forms
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext_lazy as _
@@ -8,8 +9,11 @@ class LFSFileInput(forms.FileInput):
     """
     def render(self, name, value, attrs=None):
         output = super(LFSFileInput, self).render(name, None, attrs=attrs)
-        if value and hasattr(value, "url"):
-            output = u"""<div><a href="%s" />%s</a></div>""" % (value.url, value.name) + output
+        if value:
+            if hasattr(value, "url"):
+                output = (u"""<div><a href="%s" />%s</a></div>""" % (value.url, value.name)) + output
+            elif hasattr(value, "name"):
+                output = (u"""<div>%s</div>""" % value.name) + output
 
         if value:
             trans = _(u"Delete file")
