@@ -28,6 +28,10 @@ class Order(models.Model):
 
     **Parameters**:
 
+    number
+        The unique order number of the order which is the reference for the
+        customer.
+
     voucher_number, voucher_value, voucher_tax
 
         Storing this information here assures that we have it all time, even
@@ -40,6 +44,7 @@ class Order(models.Model):
         A link to re-pay the order (e.g. for PayPal)
 
     """
+    number = models.CharField(max_length=30)
     user = models.ForeignKey(User, verbose_name=_(u"User"), blank=True, null=True)
     session = models.CharField(_(u"Session"), blank=True, max_length=100)
 
@@ -57,6 +62,7 @@ class Order(models.Model):
 
     invoice_firstname = models.CharField(_(u"Invoice firstname"), max_length=50)
     invoice_lastname = models.CharField(_(u"Invoice lastname"), max_length=50)
+    invoice_company_name = models.CharField(_(u"Invoice company name"), null=True, blank=True, max_length=100)
     invoice_line1 = models.CharField(_(u"Invoice Line 1"), null=True, blank=True, max_length=100)
     invoice_line2 = models.CharField(_(u"Invoice Line 2"), null=True, blank=True, max_length=100)
     invoice_city = models.CharField(_(u"Invoice City"), null=True, blank=True, max_length=100)
@@ -67,6 +73,7 @@ class Order(models.Model):
 
     shipping_firstname = models.CharField(_(u"Shipping firstname"), max_length=50)
     shipping_lastname = models.CharField(_(u"Shipping lastname"), max_length=50)
+    shipping_company_name = models.CharField(_(u"shipping company name"), null=True, blank=True, max_length=100)
     shipping_line1 = models.CharField(_(u"Shipping Line 1"), null=True, blank=True, max_length=100)
     shipping_line2 = models.CharField(_(u"Shipping Line 2"), null=True, blank=True, max_length=100)
     shipping_city = models.CharField(_(u"Shipping City"), null=True, blank=True, max_length=100)
@@ -104,10 +111,10 @@ class Order(models.Model):
     def __unicode__(self):
         return "%s (%s %s)" % (self.created.strftime("%x %X"), self.customer_firstname, self.customer_lastname)
 
-    def get_pay_link(self):
+    def get_pay_link(self, request):
         """Returns a pay link for the selected payment method.
         """
-        return lfs.payment.utils.get_pay_link(self.payment_method, self)
+        return lfs.payment.utils.get_pay_link(request, self.payment_method, self)
 
     def get_name(self):
         order_name = ""
@@ -146,7 +153,7 @@ class OrderItem(models.Model):
 
     @property
     def amount(self):
-        return self.product_amount
+        return self.product.get_clean_quantity(self.product_amount)
 
     def get_properties(self):
         """Returns properties of the order item. Resolves option names for
@@ -154,7 +161,7 @@ class OrderItem(models.Model):
         """
         properties = []
         for property_value in self.properties.all():
-            price = ""
+
             if property_value.property.is_select_field:
                 try:
                     option = PropertyOption.objects.get(pk=int(float(property_value.value)))
@@ -164,14 +171,14 @@ class OrderItem(models.Model):
                 else:
                     value = option.name
                     price = option.price
-            elif property_value.property.is_number_field:
+            else:
                 format_string = "%%.%sf" % property_value.property.decimal_places
                 try:
                     value = format_string % float(property_value.value)
                 except ValueError:
                     value = "%.2f" % float(property_value.value)
-            else:
-                value = property_value.value
+
+                price = ""
 
             properties.append({
                 "name": property_value.property.name,
@@ -179,8 +186,7 @@ class OrderItem(models.Model):
                 "unit": property_value.property.unit,
                 "display_price": property_value.property.display_price,
                 "value": value,
-                "price": price,
-                "obj": property_value.property
+                "price": price
             })
 
         return properties
